//
// libTeXFormulaLinux.cpp
//
// this file contains the parts of libTeXFormula, which are specific to Linux
//
//
// this file, part of libTexFormula, is released under the conditions of the zlib license.
// for details see: http://www.gzip.org/zlib/zlib_license.html
//

#ifndef WIN32
#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <string>
#include <unistd.h>
#include <fcntl.h>


#include "libTeXFormula.h"
#include "gdevdsp.h"
#include "iapi.h"

static void *gsHandle = 0;

// function pointers to GhostScript interface
PFN_gsapi_new_instance			gsapi_new_instance_runtime;
PFN_gsapi_delete_instance		gsapi_delete_instance_runtime;
PFN_gsapi_set_stdio			gsapi_set_stdio_runtime;
PFN_gsapi_set_display_callback		gsapi_set_display_callback_runtime;
PFN_gsapi_init_with_args		gsapi_init_with_args_runtime;
PFN_gsapi_run_file			gsapi_run_file_runtime;
PFN_gsapi_exit				gsapi_exit_runtime;

// loads the GS library at runtime
bool LoadGSLibrary()
{
    // Load the library
    gsHandle = dlopen("libgs.so",RTLD_LAZY);

    if (!gsHandle || dlerror())
        return false;

    // get the function pointers to the library functions
    gsapi_new_instance_runtime = (PFN_gsapi_new_instance) dlsym(gsHandle,"gsapi_new_instance");
    if (dlerror())	return false;

    gsapi_delete_instance_runtime = (PFN_gsapi_delete_instance) dlsym(gsHandle,"gsapi_delete_instance");
    if (dlerror())	return false;

    gsapi_set_stdio_runtime = (PFN_gsapi_set_stdio) dlsym(gsHandle,"gsapi_set_stdio");
    if (dlerror())	return false;

    gsapi_set_display_callback_runtime = (PFN_gsapi_set_display_callback) dlsym(gsHandle,"gsapi_set_display_callback");
    if (dlerror())	return false;

    gsapi_init_with_args_runtime = (PFN_gsapi_init_with_args) dlsym(gsHandle,"gsapi_init_with_args");
    if (dlerror())	return false;

    gsapi_run_file_runtime = (PFN_gsapi_run_file) dlsym(gsHandle,"gsapi_run_file");
    if (dlerror())	return false;

    gsapi_exit_runtime = (PFN_gsapi_exit) dlsym(gsHandle,"gsapi_exit");
    if (dlerror())	return false;

    return true;
};


// frees the Ghostscript library
void FreeGSLibrary()
{
    if (gsHandle)
        dlclose(gsHandle);
};

// creates an temporary directory
std::string CreateTempDir()
{
    std::string dirTemplate = std::string(P_tmpdir) + "/libTex.XXXXXX";
    char *buffer = new char[dirTemplate.length() + 1];
    strcpy(buffer,dirTemplate.c_str());
    char *dirPtr = mkdtemp(buffer);
    if (!dirPtr)
    {
        delete[] buffer;
        return "";
    };

    dirTemplate = dirPtr;
    delete[] buffer;

    return dirTemplate;
};

// starts a subprocess and waits until it's termination
int startProcess(const char *CommandLine, const char *workDir = NULL, std::string *stdoutString = NULL, std::string *stderrorString = NULL)
{
    int result;

    // create pipes to redirect output
    int stdoutDesc[2], stderrorDesc[2];
    pipe(stdoutDesc);
    pipe(stderrorDesc);
    int oldStderror, oldStdout;

    // redirect stdout
    oldStdout = dup(1);
    dup2(stdoutDesc[1], 1);

    // redirect stderror
    oldStderror = dup(2);
    dup2(stderrorDesc[1], 2);

    // start the process
    if (workDir)
        result =  system( ( std::string("cd ") + workDir + " && " + CommandLine).c_str());
    else
        result = system(CommandLine);

    char Buffer[10001];

    // Read stdout
    memset(Buffer,0,10001);
    fcntl(stdoutDesc[0], F_SETFL, fcntl(stderrorDesc[0], F_GETFL) |  O_NONBLOCK);
    read(stdoutDesc[0],Buffer,10000);
    if (stdoutString) *stdoutString= Buffer;

    // Read stderror
    memset(Buffer,0,10001);
    fcntl(stderrorDesc[0], F_SETFL, fcntl(stderrorDesc[0], F_GETFL) |  O_NONBLOCK);
    read(stderrorDesc[0],Buffer,10000);
    if (stderrorString) *stderrorString = Buffer;

    // restore stdout
    dup2(oldStdout, 1);

    // restore stderror
    dup2(oldStderror, 2);

    // close pipes
    close(stdoutDesc[0]);
    close(stdoutDesc[1]);
    close(stderrorDesc[0]);
    close(stderrorDesc[1]);

    return result;
};

#endif
