//
// libTeXFormulaWin32.cpp
//
// this file contains the parts of libTeXFormula, which are specific to win32
//
//
// this file, part of libTexFormula, is released under the conditions of the zlib license.
// for details see: http://www.gzip.org/zlib/zlib_license.html
//

#ifdef WIN32
#include <stdio.h>
#include <stdlib.h>

#define WIN32_LEAN_AND_MEAN 1
#include <process.h>
#include <windows.h>
#include <io.h>
#include <direct.h>
#include <string>

#include "libTeXFormula.h"
#include "gdevdsp.h"
#include "iapi.h"
#include <iostream>


#define STDIN    0
#define STDOUT   1
#define STDERROR 2

static HMODULE gsDLL = 0;

// function pointers to GhostScript Interface
PFN_gsapi_new_instance			gsapi_new_instance_runtime;
PFN_gsapi_delete_instance		gsapi_delete_instance_runtime;
PFN_gsapi_set_stdio				gsapi_set_stdio_runtime;
PFN_gsapi_set_display_callback	gsapi_set_display_callback_runtime;
PFN_gsapi_init_with_args		gsapi_init_with_args_runtime;
PFN_gsapi_run_file				gsapi_run_file_runtime;
PFN_gsapi_exit					gsapi_exit_runtime;

// loads the GS Library at runtime
bool LoadGSLibrary()
{
    // Load the DLL
    gsDLL = LoadLibraryA("gsdll32.dll");

    if (!gsDLL)
        return false;

    // get the Function Pointers to the DLL functions
    if (!(gsapi_new_instance_runtime = (PFN_gsapi_new_instance) GetProcAddress(gsDLL,"gsapi_new_instance")))
        return false;
    if (!(gsapi_delete_instance_runtime = (PFN_gsapi_delete_instance) GetProcAddress(gsDLL,"gsapi_delete_instance")))
        return false;
    if (!(gsapi_set_stdio_runtime = (PFN_gsapi_set_stdio) GetProcAddress(gsDLL,"gsapi_set_stdio")))
        return false;
    if (!(gsapi_set_display_callback_runtime = (PFN_gsapi_set_display_callback) GetProcAddress(gsDLL,"gsapi_set_display_callback")))
        return false;
    if (!(gsapi_init_with_args_runtime = (PFN_gsapi_init_with_args) GetProcAddress(gsDLL,"gsapi_init_with_args")))
        return false;
    if (!(gsapi_run_file_runtime = (PFN_gsapi_run_file) GetProcAddress(gsDLL,"gsapi_run_file")))
        return false;
    if (!(gsapi_exit_runtime = (PFN_gsapi_exit) GetProcAddress(gsDLL,"gsapi_exit")))
        return false;

    return true;
};

// creates an temporary directory
std::string CreateTempDir()
{
    char *tempName = _tempnam(NULL, "libTex.");
    if (!tempName)
        return "";

    if (!CreateDirectoryA(tempName,NULL))
        return "";

    return tempName;
};

// frees the Ghostscript library
void FreeGSLibrary()
{
    if (gsDLL)
        FreeLibrary(gsDLL);
};

// starts a subprocess and waits until it's termination
int startProcess(const char *CommandLine, const char *workDir = NULL, std::string *stdoutString = NULL, std::string *stderrorString = NULL)
{
    DWORD returnCode;
    STARTUPINFOA si;
    SECURITY_ATTRIBUTES sa;
    PROCESS_INFORMATION pi;
    HANDLE stdoutReadPipe,stdoutWritePipe;
    HANDLE stderrorReadPipe,stderrorWritePipe;

    // Create pipe for STDERR
    sa.nLength=sizeof(sa);
    sa.bInheritHandle=TRUE;
    sa.lpSecurityDescriptor=NULL;
    CreatePipe(&stdoutReadPipe,&stdoutWritePipe,&sa,1024);
    CreatePipe(&stderrorReadPipe,&stderrorWritePipe,&sa,1024);

    // Prepare si structure
    ZeroMemory( &si, sizeof(si) );
    si.wShowWindow = SW_HIDE;
    si.dwFlags = STARTF_USESHOWWINDOW |  STARTF_USESTDHANDLES;
    si.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
    si.hStdOutput = stdoutWritePipe;
    si.hStdError = stderrorWritePipe;
    ZeroMemory( &pi, sizeof(pi) );

    // Start the child process.
    if( !CreateProcessA(
                NULL,								  // No module name (use command line).
                const_cast<LPSTR>(CommandLine),       // Command line.
                NULL,								  // Process handle not inheritable.
                NULL,								  // Thread handle not inheritable.
                TRUE,								  // Set handle inheritance to FALSE.
                CREATE_NO_WINDOW | DETACHED_PROCESS,  // Don't open Console
                NULL,								  // Use parent's environment block.
                const_cast<LPSTR>(workDir),		      // Use parent's starting directory.
                &si,								  // Pointer to STARTUPINFO structure.
                &pi )								  // Pointer to PROCESS_INFORMATION structure.
      )
        return -1;

    // Wait until child process exits
    WaitForSingleObject( pi.hProcess, INFINITE );


    char Buffer[10001];
    DWORD bytes = 0;
    DWORD mode = PIPE_READMODE_BYTE|PIPE_NOWAIT;

    // Read STDOUT
    memset(Buffer,0,10001);
    SetNamedPipeHandleState(stdoutReadPipe,&mode, NULL, NULL);
    ReadFile(stdoutReadPipe,Buffer,10000,&bytes,NULL);
    if (stdoutString) *stdoutString= Buffer;

    // Read STDERROR
    memset(Buffer,0,10001);
    SetNamedPipeHandleState(stderrorReadPipe,&mode, NULL, NULL);
    ReadFile(stderrorReadPipe,Buffer,10000,&bytes,NULL);
    if (stderrorString) *stderrorString = Buffer;

    // Get processes exit code
    GetExitCodeProcess(pi.hProcess,&returnCode);

    // Close process and thread handles
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );
    CloseHandle( stdoutReadPipe );
    CloseHandle( stdoutWritePipe );
    CloseHandle( stderrorReadPipe );
    CloseHandle( stderrorWritePipe );

    return returnCode;
};

#endif
